/* Materiel et lumieres                         */
/*                                              */
/* Auteur: Nicolas JANEY                        */
/* nicolas.janey@univ-fcomte.fr                 */
/* Fevrier 2021                                 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>

/* Constantes globales                          */

static const float blanc[] = { 1.0F,1.0F,1.0F,1.0F };
static const float gris[] = { 0.5F,0.5F,0.5F,1.0F };
static const float noir[] = { 0.0F,0.0F,0.0F,1.0F };

/* Variables globales                           */

static int cFond = 0;              // Numero de la couleur de fond (0: gris, 1: blanc, 2:noir)
static float r1 = 0.0F;            // Angle de rotation de la scene sur elle-meme
static float r2 = 0.0F;            // Angle de rotation de la scene sur elle-meme
static int anim = 1;               // Flag d'action/desactivation de l'animation
static int pMode = 1;              // Flag de switch entre modes d'affichage wireframe et fill 
static int wTx = 640;              // Resolution horizontale de la fenetre
static int wTy = 480;              // Resolution verticale de la fenetre
static int wPx = 50;               // Position horizontale de la fenetre
static int wPy = 50;               // Position verticale de la fenetre
static double sz = 0.5;            // Taille facettes

/* Fonction d'initialisation des parametres     */
/* OpenGL ne changeant pas au cours de la vie   */
/* du programme                                 */

static void init(void) {
  glDepthFunc(GL_LESS);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
  glEnable(GL_AUTO_NORMAL);
}

/* Scene dessinee                               */

static void solidSphere(double rayon,double hauteur,double sz) {
  int n =(int) (rayon*2*3.14159/sz);
  if ( n < 3)
    n = 3;
  int m =(int) (hauteur/2.0*3.14159/sz);
  if ( m < 3)
    m = 3;
  glPushMatrix();
  glRotatef(90.0F,1.0F,0.0F,0.0F);
  glScalef(rayon,rayon,hauteur/2.0);
  glutSolidSphere(1.0,n,m);
  glPopMatrix();
}

static void element() {
  glPushMatrix();
  glTranslatef(-5.0F,0.0F,5.0F);
  solidSphere(1.5,20.0,sz);
  for ( int i = 0 ; i < 2 ; i++ ) {
    glTranslatef(0.0F,5.0F,0.0F);
    solidSphere(2.5,5.0,sz);
    glRotatef(-90.0F,0.0F,0.0F,1.0F);
    glTranslatef(0.0F,5.0F,0.0F);
    solidSphere(1.5,20.0,sz); }
  glPopMatrix();
}

static void scene() {
  glPushMatrix();
  for ( int i = 0 ; i < 4 ; i++ ) {
    glPushMatrix();
    glRotatef(90.0F*i,1.0F,0.0F,0.0F);
    element();
    glPopMatrix(); }
  glPopMatrix();
}

/* Fonction executee lorsqu'aucun evenement     */
/* n'est en file d'attente                      */

static void idle(void) {
  r1 += 1.1355F;
  r2 += 0.6755F;
  glutPostRedisplay();
}

/////////////////////////////////////////////////////////////

static int l = 0;
static int ldef = 0;
static int l0 = 0;
static int l1 = 0;
static int l2 = 0;
static int l3 = 0;
static int ms = 0;
static int md = 0;
static int ma = 0;
static int me = 0;

static void activationLumieres(void) {
  if ( l )
    glEnable(GL_LIGHTING);
    else
    glDisable(GL_LIGHTING);
  if ( l0 )
    glEnable(GL_LIGHT0);
    else
    glDisable(GL_LIGHT0);
  if ( l1 )
    glEnable(GL_LIGHT1);
    else
    glDisable(GL_LIGHT1);
  if ( l2 )
    glEnable(GL_LIGHT2);
    else
    glDisable(GL_LIGHT2);
  if ( l3 )
    glEnable(GL_LIGHT3);
    else
    glDisable(GL_LIGHT3);
}

static void configurationLumieres(void) {
  const GLfloat magenta[]     = { 1.0F,0.0F,1.0F,1.0F };
  const GLfloat rouge[]       = { 1.0F,0.0F,0.0F,1.0F };
  const GLfloat vert[]        = { 0.0F,1.0F,0.0F,1.0F };
  const GLfloat bleu[]        = { 0.0F,0.0F,1.0F,1.0F };
  const GLfloat jaune[]       = { 1.0F,1.0F,0.0F,1.0F };
  const GLfloat grisMoyen[]   = { 0.5F,0.5F,0.5F,1.0F };
  const GLfloat pos0[] = { 2.0F,2.0F,2.0F,1.0F };
  glLightfv(GL_LIGHT0,GL_POSITION,pos0);
  glLightfv(GL_LIGHT0,GL_DIFFUSE,rouge);
  glLightfv(GL_LIGHT0,GL_SPECULAR,noir);
  glLightfv(GL_LIGHT0,GL_AMBIENT,noir);
  const GLfloat pos1[] = { -2.0F,0.0F,4.0F,0.0F };
  glLightfv(GL_LIGHT1,GL_POSITION,pos1);
  glLightfv(GL_LIGHT1,GL_DIFFUSE,vert);
  glLightfv(GL_LIGHT1,GL_SPECULAR,blanc);
  glLightfv(GL_LIGHT1,GL_AMBIENT,noir);
  const GLfloat pos2[] = { -3.0F,-3.0F, 15.0F,1.0F };
  const GLfloat dir2[] = {  3.0F, 3.0F,-10.0F };
  glLightfv(GL_LIGHT2,GL_POSITION,pos2);
  glLightfv(GL_LIGHT2,GL_SPOT_DIRECTION,dir2);
  glLightf(GL_LIGHT2,GL_SPOT_CUTOFF,20.0F);
  glLightfv(GL_LIGHT2,GL_DIFFUSE,bleu);
  glLightfv(GL_LIGHT2,GL_SPECULAR,jaune);
  glLightfv(GL_LIGHT2,GL_AMBIENT,noir);
  const GLfloat pos3[] = {  0.0F,0.0F,0.0F,1.0F };
  glLightfv(GL_LIGHT3,GL_POSITION,pos3);
  glLightfv(GL_LIGHT3,GL_DIFFUSE,noir);
  glLightfv(GL_LIGHT3,GL_SPECULAR,noir);
  glLightfv(GL_LIGHT3,GL_AMBIENT,magenta);
}

static void configurationMateriel(void) {
  const GLfloat diffuseDefaut[]  = { 0.8F,0.8F,0.8F,1.0F };
  const GLfloat ambientDefaut[]  = { 0.2F,0.2F,0.2F,1.0F };
  const GLfloat specularDefaut[] = { 0.0F,0.0F,0.0F,1.0F };
  const GLfloat emissionDefaut[] = { 0.0F,0.0F,0.0F,1.0F };
  const GLfloat magenta[]        = { 1.0F,0.0F,1.0F,1.0F };
  const GLfloat cyan[]           = { 0.0F,1.0F,1.0F,1.0F };
  const GLfloat jaune[]          = { 1.0F,1.0F,0.0F,1.0F };
  if ( md ) {
    glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,magenta); }
    else {
    glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,diffuseDefaut); }
  if ( ma ) {
    glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,cyan); }
    else {
    glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,ambientDefaut); }
  if ( me ) {
    glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,jaune); }
    else {
    glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,emissionDefaut); }
  if ( ms ) {
    glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,64.0);
    glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,blanc); }
    else {
    glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,0.0);
    glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,specularDefaut); }
}

/* Fonction executee lors d'un rafraichissement */
/* de la fenetre de dessin                      */

static void display(void) {
  const float *fond;
  switch (cFond) {
    case 0 :
      fond = gris;
      break;
    case 1 :
      fond = blanc;
      break;
    case 2 :
      fond = noir;
      break; }
  glClearColor(fond[0],fond[1],fond[2],fond[3]);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glPolygonMode(GL_FRONT_AND_BACK,(pMode == 1) ? GL_FILL : GL_LINE);
  glPushMatrix();
  glPushAttrib(GL_LIGHTING_BIT);
  int notReset = 1;
  activationLumieres();
  configurationLumieres();
  configurationMateriel();
  if ( ldef ) {
    notReset = 0;
    glPopAttrib();
    activationLumieres(); }
  glRotatef(r1,1.0F,2.0F,3.0F);
  glRotatef(r2,-1.0F,2.0F,-3.0F);
  scene();
  if ( notReset ) {
    glPopAttrib(); }
  glPopMatrix();
  glFlush();
  glutSwapBuffers();
  int error = glGetError();
  if ( error != GL_NO_ERROR )
    printf("Attention erreur %d\n",error);
}

/* Fonction executee lors de l'appui            */
/* d'une touche alphanumerique du clavier       */

static void keyboard(unsigned char key,int x,int y) {
  switch (key) {
    case 0x0D :
      l = !l;
      glutPostRedisplay(); 
      break;
    case 43 :
      sz /= 1.01;
      glutPostRedisplay(); 
      break;
    case 45 :
      sz *= 1.01;
      glutPostRedisplay(); 
      break;
    case 'a' :
      { anim = !anim;
        glutIdleFunc(( anim ) ? idle : NULL); }
      break;
    case 'z' :
      { pMode = !pMode;
        glutPostRedisplay(); }
      break;
    case 'f' :
      { cFond = (cFond+1)%3;
        glutPostRedisplay(); }
      break;
    case 'F' :
      { static int full = 0;
        static int wFx;
        static int wFy;
        full = !full;
        if ( full ) {
          wFx = glutGet(GLUT_WINDOW_WIDTH);
          wFy = glutGet(GLUT_WINDOW_HEIGHT);
          wPx = glutGet(GLUT_WINDOW_X);
          wPy = glutGet(GLUT_WINDOW_Y);
          glutFullScreen(); }
          else {
          glutPositionWindow(wPx,wPy);
          glutReshapeWindow(wFx,wFy); } }
      break;
    case 0x1B :
      exit(0);
      break; }
}

/* Fonction executee lors de l'appui            */
/* d'une touche speciale du clavier :           */
/*   - touches de curseur                       */
/*   - touches de fonction                      */

static void special(int specialKey,int x,int y) {
  switch(specialKey) {
    case GLUT_KEY_F5 :
      ms = !ms;
      glutPostRedisplay();
      break;
    case GLUT_KEY_F6 :
      md = !md;
      glutPostRedisplay();
      break;
    case GLUT_KEY_F7 :
      ma = !ma;
      glutPostRedisplay();
      break;
    case GLUT_KEY_F8 :
      me = !me;
      glutPostRedisplay();
      break;
    case GLUT_KEY_F1 :
      l0 = !l0;
      glutPostRedisplay();
      break;
    case GLUT_KEY_F2 :
      l1 = !l1;
      glutPostRedisplay();
      break;
    case GLUT_KEY_F3 :
      l2 = !l2;
      glutPostRedisplay();
      break;
    case GLUT_KEY_F4 :
      l3 = !l3;
      glutPostRedisplay();
      break;
    case GLUT_KEY_F10 :
      ldef = !ldef;
      glutPostRedisplay();
      break;
    case GLUT_KEY_UP :
      r1 += 1.0F;
      glutPostRedisplay();
      break;
    case GLUT_KEY_DOWN :
      r1 -= 1.0F;
      glutPostRedisplay();
      break;
    case GLUT_KEY_RIGHT :
      r2 += 1.0F;
      glutPostRedisplay();
      break;
    case GLUT_KEY_LEFT :
      r2 -= 1.0F;
      glutPostRedisplay();
      break; }
}

/////////////////////////////////////////////////////////////

/* Fonction executee lors d'un changement       */
/* de la taille de la fenetre OpenGL            */

static void reshape(int wx,int wy) {
  wTx = wx; 
  wTy = wy; 
  glViewport(0,0,wx,wy); 
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  if ( wy < wx )
    gluPerspective(80.0,(double) wx/wy,20.0-15.0,20.0+15.0);
    else
    gluPerspective(80.0*((double) wy/wx),(double) wx/wy,20.0-15.0,20.0+15.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0.0,0.0,20.0,0.0,0.0,0.0,0.0,1.0,0.0);
}

/* Fonction principale                          */

int main(int argc,char **argv) {
  glutInit(&argc,argv);
  glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE);
  glutInitWindowSize(wTx,wTy);
  glutInitWindowPosition(wPx,wPy);
  glutCreateWindow("Materiel et lumires");
  init();
  glutSpecialFunc(special);
  glutKeyboardFunc(keyboard);
  glutReshapeFunc(reshape);
  glutIdleFunc((anim) ? idle : NULL);
  glutDisplayFunc(display);
  glutMainLoop();
  return(0);
}
