/* Placage de 6 textures sur les faces          */
/* d'un cube                                    */
/*                                              */
/* Auteur: Nicolas JANEY                        */
/* nicolas.janey@univ-fcomte.fr                 */
/* Fevrier 2021                                 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "Maths\Pos3D.h"
#include "Maths\Dir3D.h"
#include "Maths\TG3D.h"
#include "Maths\Rt3D.h"
#include "Maths\TR3D.h"

#include "Bille.h"
#include "ObjetsOpenGL.h"

#ifndef M_PI
#define M_PI 3.14159265358979323846264338327950288
#endif

/* Variables et constantes globales             */

static const float blanc[]       = { 1.0F,1.0F,1.0F,1.0F };

static int affichage = 1;
static int texture = 1;
static int animation = 0;

static float ry = 60.0F;

static float inclinaison = 2.0F;
static Bille bille;

//////////////////////////////////////////////////

#include "PNG/ChargePngFile.h"

static unsigned int textureID[6] = { 0,0,0,0,0,0 };

/* Fonction de chargement d'une texture         */
/* a partir d'un fichier image au format png    */
/* filename : le nom du fichier                 */
/* textureID : le handle de la texture          */

static void chargementTexture(char *filename,unsigned int textureID) {
  glBindTexture(GL_TEXTURE_2D,textureID);
  glPixelStorei(GL_UNPACK_ALIGNMENT,1); 
  { int rx;
    int ry;
    unsigned char *img = chargeImagePng(filename,&rx,&ry);
    if ( img ) {
      glTexImage2D(GL_TEXTURE_2D,0,3,rx,ry,0,GL_RGB,GL_UNSIGNED_BYTE,img);
      free(img);
      printf("Texture charge %d : %s\n",textureID,filename); }
      else {
      printf("Texture non charge\n"); } }
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT); 
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); 
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); 
}

/* Fonction de chargement d'une texture         */
/* a partir d'un fichier image au format png    */
/* filename : le nom du fichier                 */
/* textureID : le handle de la texture          */

static unsigned int chargementTexture(char *filename) {
  unsigned int textureID;
  glGenTextures(1,&textureID);
  if ( !textureID ) {
    printf("Texture non charge\n");
    return 0; }
  glBindTexture(GL_TEXTURE_2D,textureID);
  glPixelStorei(GL_UNPACK_ALIGNMENT,1); 
  int rx;
  int ry;
  unsigned char *img = chargeImagePng(filename,&rx,&ry);
  if ( img ) {
    glTexImage2D(GL_TEXTURE_2D,0,3,rx,ry,0,GL_RGB,GL_UNSIGNED_BYTE,img);
    free(img);
    printf("Texture charge %d : %s\n",textureID,filename);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT); 
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); 
    glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
    return textureID; }
    else {
    printf("Texture non charge\n");
    return 0; }
}

/* Fonction d'initialisation des parametres     */
/* OpenGL ne changeant pas au cours de la vie   */
/* du programme                                 */

static void initGL(void) {
  glClearColor(0.25F,0.25F,0.25F,1.0F);
  glLightfv(GL_LIGHT0,GL_DIFFUSE,blanc);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glDepthFunc(GL_LESS);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);
}

/* Scene dessinee                               */

static void scene(void) {
  glPushMatrix();
  glPushMatrix();
  glRotatef(inclinaison,1.0F,0.0F,0.0F);
  glTranslatef(0.0F,-0.1F,0.0F);
  glScalef(10.0F,0.2F,20.0F);
  solidCube(1.0,textureID);
  glPopMatrix();
  glPushMatrix();
  glTranslatef(bille.p.x,bille.p.y,bille.p.z);
  glRotatef(bille.rt,1.0F,0.0F,0.0F);
  solidSphere(1.0,36,36,bille.textureID);
  glPopMatrix();
  glPopMatrix();
}

/* Desallocation des textures                   */

static void clean(void) {
  for ( int i = 0 ; i < 6 ; i++ )
    if ( textureID[i] != 0 )
      glDeleteTextures(1,&textureID[i]);
  if ( bille.textureID != 0 )
    glDeleteTextures(1,&bille.textureID);
}

//////////////////////////////////////////////////

/* Fonction executee lors de l'appui            */
/* d'une touche speciale du clavier :           */
/*   - touches de curseur                       */
/*   - touches de fonction                      */

static void special(int specialKey,int x,int y) {
  switch(specialKey) {
    case GLUT_KEY_RIGHT :
      ry += 1.0F;
      glutPostRedisplay();
      break;
    case GLUT_KEY_LEFT :
      ry -= 1.0F;
      glutPostRedisplay();
      break; }
}

/* Fonction executee lors d'un rafraichissement */
/* de la fenetre de dessin                      */

static void display(void) {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  const GLfloat light0_position[] = { 0.0,0.0,10.0,1.0 };
  glPolygonMode(GL_FRONT_AND_BACK,(affichage) ? GL_FILL : GL_LINE);
  if ( texture ) 
    glEnable(GL_TEXTURE_2D);
    else
    glDisable(GL_TEXTURE_2D);
  glPushMatrix();
  glLightfv(GL_LIGHT0,GL_POSITION,light0_position);
  gluLookAt(0.0,0.0,100.0,5.0,-20.0,0.0,0.0,1.0,0.0);
  glRotatef(ry,0.0F,1.0F,0.0F);
  scene();
  glPopMatrix();
  glFlush();
  glutSwapBuffers();
  int error = glGetError();
  if ( error != GL_NO_ERROR )
    printf("Erreur OpenGL: %d\n",error);
}

/* Fonction executee lors d'un changement       */
/* de la taille de la fenetre OpenGL            */
/* -> Ajustement de la camera de visualisation  */

static void reshape(int tx,int ty) {
  glViewport(0,0,tx,ty); 
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  double ratio =(double) tx/ty;
  if ( ratio >= 1.0 )
    gluPerspective(25.0,ratio,0.01,1000.0);
    else
    gluPerspective(25.0/ratio,ratio,0.01,1000.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

/* Fonction executee en tache de fond           */

static void idle(void) {
  if ( bille.dp < 19.0F ) {
    double v = bille.vitesse();
    bille.p.x += bille.v.x;
    bille.p.y += bille.v.y;
    bille.p.z += bille.v.z;
    bille.dp += v;
    bille.vrt = 360.0F*bille.dp/2.0/M_PI-bille.rt;
    bille.rt += bille.vrt;
    double energieCinetique = v*v;
    double deltaEnergiePotentielle = -bille.v.y/100.0;
    double nouvelleEnergieCinetique = energieCinetique + deltaEnergiePotentielle;
    double facteurVitesse = sqrt(nouvelleEnergieCinetique)/v;
  //  printf("%12.6lf %12.6lf %12.6lf %12.6lf\n",v,bille.dp,sqrt(bille.dp),sqrt(bille.dp)-old);
    bille.v.y *= facteurVitesse;
    bille.v.z *= facteurVitesse;
    glutPostRedisplay(); }
    else {
    double v = bille.vitesse();
    bille.p.x += bille.v.x;
    bille.p.y += bille.v.y;
    bille.p.z += bille.v.z;
    bille.dp += v;
    bille.rt += bille.vrt;
    double energieCinetique = v*v;
    double deltaEnergiePotentielle = -bille.v.y/100.0;
    double nouvelleEnergieCinetique = energieCinetique + deltaEnergiePotentielle;
    bille.v.y = -sqrt(nouvelleEnergieCinetique-bille.v.z*bille.v.z);
    glutPostRedisplay(); }
}

/* Fonction executee lors de la frappe          */
/* d'une touche du clavier                      */

static void keyboard(unsigned char key,int x,int y) {
  switch ( key ) {
    case 'a' :
      animation = !animation;
      glutIdleFunc(( animation ) ? idle : NULL);
      break;
    case 0x0D :
      affichage = (affichage+1)%2;
      glutPostRedisplay(); 
      break;
    case 0x20 :
      texture = (texture+1)%2;
      glutPostRedisplay(); 
      break;
    case 0x1B :
      exit(0); 
      break; }
}

/* Fonction d'initialisation de la scene        */

static void initScene(void) {
  glGenTextures(6,textureID);
  chargementTexture("Bois.png",textureID[0]);
  chargementTexture("Bois.png",textureID[1]);
  chargementTexture("Bois.png",textureID[2]);
  chargementTexture("Bois.png",textureID[3]);
  chargementTexture("Bois.png",textureID[4]);
  chargementTexture("Bois.png",textureID[5]);
  Dir3D a(1.0F,0.0F,0.0F);
  Rt3D r(inclinaison,a);
  Tr3D t(0.0F,1.0F,-9.0F);
  TG3D tg;
  tg = tg.mult(r);
  tg = tg.mult(t);
  Pos3D pos;
  bille.p = pos.mult(tg);
  double angle = -inclinaison*M_PI/180.0;
  bille.v.x = 0.0F;
  bille.v.y = 0.01F * sin(angle);
  bille.v.z = 0.01F * cos(angle);
  bille.textureID = chargementTexture("Earth.png");
}

/* Fonction principale                          */

int main(int argc,char **argv) {

  atexit(clean);

  glutInit(&argc,argv);
  glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE);
  glutInitWindowSize(640,960); 
  glutInitWindowPosition(50,50); 
  glutCreateWindow("Une bille sur un plan inclin"); 
  initGL();
  initScene();
  glutSpecialFunc(special);
  glutKeyboardFunc(keyboard);
  glutReshapeFunc(reshape);
  glutDisplayFunc(display);
  glutMainLoop();
  return(0);
}
