///
/// \file PotentiometreCirculaire.h
/// Classe mere du composant PotentiometreCirculaire.
/// 
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 01/01/10
///

#ifndef ____POTENTIOMETRECIRCULAIRE____
#define ____POTENTIOMETRECIRCULAIRE____

#include <stdio.h>

#include "Controle.h"

class Couleur;

///
/// \class PotentiometreCirculaire
/// Classe mere du composant PotentiometreCirculaire.
/// 
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 01/01/10
///

class PotentiometreCirculaire : public Controle {

  protected :
    int rayon;                        ///< Rayon d'affichage.
    float min;                        ///< Valeur minimale.
    float max;                        ///< Valeur maximale.
    float *v;                         ///< Variable geree.
    float save;                       ///< Sauvegarde de la variable geree (pour annulation).
    char titre[512];                  ///< Titre.
    Couleur *coulTexte;               ///< Couleur du texte.
    Couleur *coulTexteBouton;         ///< Couleur du texte du bouton.
    Couleur *coulBouton;              ///< Couleur du bouton.

  public :

///
/// Constructeur basique.
///
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    PotentiometreCirculaire(void);

///
/// Constructeur avec parametres d'initialisation.
///
/// @param px La position en x du PotentiometreCirculaire.
/// @param py La position en y du PotentiometreCirculaire.
/// @param rayon Le rayon du PotentiometreCirculaire.
/// @param coul La couleur de base du PotentiometreCirculaire.
/// @param min La valeur minimale du PotentiometreCirculaire.
/// @param max La valeur maximale du PotentiometreCirculaire.
/// @param coulB La couleur de dessin.
/// @param titre La chaine de caracteres du PotentiometreCirculaire.
/// @param coulT La couleur de dessin du texte.
/// @param coulTB La couleur de dessin du PotentiometreCirculaire.
/// @param v La valeur geree par le PotentiometreCirculaire.
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    PotentiometreCirculaire(int px,int py,int rayon,Couleur *coul,float min,float max,Couleur *coulB,const char *titre,Couleur *coulT,Couleur *coulTB,float *v);

///
/// Destructeur.
///
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    ~PotentiometreCirculaire(void);

///
/// Dessin du PotentiometreCirculaire.
///
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    virtual void dessiner(void);

///
/// Sauvegarde des valeurs actuelles liees au PotentiometreCirculaire.
///
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    virtual void sauve(void);

///
/// Restoration des valeurs sauvegardees liees au PotentiometreCirculaire.
///
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    virtual void annule(void);

///
/// Selection du PotentiometreCirculaire lie a la position (x,y).
///
/// @param x La coordonnee x.
/// @param y La coordonnee y.
/// @return Le booleen indiquant que la selection a bien ete geree.
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    virtual int selection(int x,int y);

///
/// Modification du PotentiometreCirculaire lie a la position (x,y) et a un vecteur (dx,dy).
///
/// @param x La coordonnee x.
/// @param y La coordonnee y.
/// @param dx La valeur dx.
/// @param dy La valeur dy.
/// @return Le booleen indiquant que la selection a bien ete geree.
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    virtual int gereModification(int x,int y,int dx,int dy);

///
/// Sauvegarde du PotentiometreCirculaire dans un fichier.
///
/// @param f Le fichier de sauvegarde.
/// @author Nicolas Janey
/// @author nicolas.janey@univ-fcomte.fr
/// @version 1.0, 25/02/10
///
    virtual void sauve(FILE *f);
};

#endif
