/**
 * \file coordonn.cpp
 * Structures et fonctions pour la manipulation de coordonnees.
 * 
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 01/01/10
 */

#include <math.h>

#include "coordonn.h"

int egal(coord_4D *c1,coord_4D *c2) {
  return( ( fabs(c1->x-c2->x) < 0.0001 ) &&
          ( fabs(c1->y-c2->y) < 0.0001 ) &&
          ( fabs(c1->z-c2->z) < 0.0001 ) );
}

void normalise(coord_4D *v) {
  double d = sqrt(v->x*v->x+v->y*v->y+v->z*v->z);
  if ( d < 0.00001 )
    return;
  v->x /= d;
  v->y /= d;
  v->z /= d;
}

void normale(coord_4D *c1,coord_4D *c2,coord_4D *c3,coord_4D *n) {
  coord_4D v1 = { c2->x-c1->x,c2->y-c1->y,c2->z-c1->z,0.0F };
  coord_4D v2 = { c3->x-c1->x,c3->y-c1->y,c3->z-c1->z,0.0F };
  n->x = v1.y*v2.z-v2.y*v1.z;
  n->y = v1.z*v2.x-v2.z*v1.x;
  n->z = v1.x*v2.y-v2.x*v1.y;
  n->t = 0.0;
  normalise(n);
}

int egal(coord_3D *c1,coord_3D *c2) {
  return( ( fabs(c1->x-c2->x) < 0.0001 ) &&
          ( fabs(c1->y-c2->y) < 0.0001 ) &&
          ( fabs(c1->z-c2->z) < 0.0001 ) );
}

void normalise(coord_3D *v) {
  double d = sqrt(v->x*v->x+v->y*v->y+v->z*v->z);
  if ( d < 0.00001 )
    return;
  v->x =(__TYPE__) (v->x/d);
  v->y =(__TYPE__) (v->y/d);
  v->z =(__TYPE__) (v->z/d);
}

void normale(coord_3D *c1,coord_3D *c2,coord_3D *c3,coord_3D *n) {
  coord_3D v1 = { c2->x-c1->x,c2->y-c1->y,c2->z-c1->z };
  coord_3D v2 = { c3->x-c1->x,c3->y-c1->y,c3->z-c1->z };
  n->x = v1.y*v2.z-v2.y*v1.z;
  n->y = v1.z*v2.x-v2.z*v1.x;
  n->z = v1.x*v2.y-v2.x*v1.y;
  normalise(n);
}

int egal(coord_3D_court *c1,coord_3D_court *c2) {
  return( ( fabs(c1->x-c2->x) < 0.0001 ) &&
          ( fabs(c1->y-c2->y) < 0.0001 ) &&
          ( fabs(c1->z-c2->z) < 0.0001 ) );
}

void normalise(coord_3D_court *v) {
  double d = sqrt(v->x*v->x+v->y*v->y+v->z*v->z);
  if ( d < 0.00001 )
    return;
  v->x =(__TYPE__) (v->x/d);
  v->y =(__TYPE__) (v->y/d);
  v->z =(__TYPE__) (v->z/d);
  if ( v->x > 1.0F ) {
    v->x = 1.0F;
    v->y = 0.0F;
    v->z = 0.0F;
    return; }
  if ( v->x < -1.0F ) {
    v->x = -1.0F;
    v->y = 0.0F;
    v->z = 0.0F;
    return; }
  if ( v->y > 1.0F ) {
    v->y = 1.0F;
    v->x = 0.0F;
    v->z = 0.0F;
    return; }
  if ( v->y < -1.0F ) {
    v->y = -1.0F;
    v->x = 0.0F;
    v->z = 0.0F;
    return; }
  if ( v->z > 1.0F ) {
    v->z = 1.0F;
    v->x = 0.0F;
    v->y = 0.0F;
    return; }
  if ( v->z < -1.0F ) {
    v->z = -1.0F;
    v->x = 0.0F;
    v->y = 0.0F;
    return; }
}

void normale(coord_3D_court *c1,coord_3D_court *c2,coord_3D_court *c3,coord_3D_court *n) {
  coord_3D_court v1 = { c2->x-c1->x,c2->y-c1->y,c2->z-c1->z };
  coord_3D_court v2 = { c3->x-c1->x,c3->y-c1->y,c3->z-c1->z };
  n->x = v1.y*v2.z-v2.y*v1.z;
  n->y = v1.z*v2.x-v2.z*v1.x;
  n->z = v1.x*v2.y-v2.x*v1.y;
  normalise(n);
}

void normale(coord_3D_court *c1,coord_3D_court *c2,coord_3D_court *c3,coord_3D *n) {
  coord_3D_court v1 = { c2->x-c1->x,c2->y-c1->y,c2->z-c1->z };
  coord_3D_court v2 = { c3->x-c1->x,c3->y-c1->y,c3->z-c1->z };
  n->x = v1.y*v2.z-v2.y*v1.z;
  n->y = v1.z*v2.x-v2.z*v1.x;
  n->z = v1.x*v2.y-v2.x*v1.y;
  normalise(n);
}
