#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "mathemat.h"
#include "max_min.h"
#include "coordonn.h"
#include "mathmath.h"

static unsigned long        Seed = 1 ;

/* ************************************************************************* */
/*                      MODULE DE GESTION MATHEMATIQUE                       */
/* ************************************************************************* */

double coefficient_intersection(struct coord_2D *p1,struct coord_2D *p2,struct coord_2D *pa,struct coord_2D *pb)
  { double                   s2 ;
    struct coord_2D          n,pa1,pba ;

    n.x = p2->y - p1->y ;
    n.y = p1->x - p2->x ;
    pa1.x = pa->x - p1->x ;
    pa1.y = pa->y - p1->y ;
    pba.x = pb->x - pa->x ;
    pba.y = pb->y - pa->y ;
    s2 = n.x * pba.x + n.y * pba.y ;
    return(( fabs(s2) > 0.001 ) ? -(n.x * pa1.x + n.y * pa1.y) / s2 : 1000000.) ;
}
/* ------------------------------------------------------------------------ */
void echange_entier(int *i,int *j)
  { int                aux ;

    aux = *i ;
    *i = *j ;
    *j = aux ;
}
/* ------------------------------------------------------------------------ */
void echange_double(double *i,double *j)
  { double             aux ;

    aux = *i ;
    *i = *j ;
    *j = aux ;
}
/* ------------------------------------------------------------------------ */
double rand_0_1(void)
  { double    a ;

    a =(double) my_rand(32000)/3001 ;
    return(a-floor(a)) ;
}
/* ------------------------------------------------------------------------ */
unsigned my_rand(unsigned limite)
  { long lim,quo ;
    Seed = 0x015a4e35L * Seed + 1 ;
    lim =(Seed>>8)&0x00FFFFFFL ;
    quo = lim/limite ;
    if ( lim < 0 )
      quo-- ;
    unsigned r =(unsigned) ((unsigned long) lim-(unsigned long) quo*(unsigned long) limite) ;
    return(r);
}
/* ------------------------------------------------------------------------ */
double modulo(double x,double y)
  { return(x-floor(x/y)*y) ;
}
/* ------------------------------------------------------------------------ */
double degreversradian(double angle)
  { return(angle*M_PI/180.) ;
}
/* ------------------------------------------------------------------------ */
unsigned my_rand_double(double limite,double vp)
  { double        v ;

    v = limite+M_PI+vp ;
    v -= floor(v) ;
    v *= 100 ;
    v -= floor(v) ;
    return((unsigned) (v*limite));
}
/* ------------------------------------------------------------------------ */
long my_rand_long(long limite)
  { long            lim,quo ;

    Seed = 0x015a4e35L * Seed + 1 ;
    lim =(Seed>>8)&0x00FFFFFFL ;
    quo = lim/limite ;
    if ( lim < 0 )
      quo-- ;
    return(lim-quo*limite);
}
/* ------------------------------------------------------------------------ */
double my_rand_lf(double limite)
  { double        v ;
    long          r ;

    r = my_rand(10001) ;
    v =(double) r / 10000.0 ;
    return(v*limite);
}
/* ------------------------------------------------------------------------ */
void init_rand(unsigned seed)
  { Seed = seed ;
}
/* ------------------------------------------------------------------------ */
/*   La fonction hasard rend une valeur entire comprise entre 0 et 10000   */
/* ------------------------------------------------------------------------ */
int hasard (void)
  { return ( my_rand(10001) ) ;
}
/* ------------------------------------------------------------------------ */
/*         Fonction calculant un nombre affect d'une incertitude           */
/* ------------------------------------------------------------------------ */
double incertitude(double interval,double val,int decision)
  { return (( !decision ) ? val : (((double) my_rand(100) - 50.) / 100. * interval + 1.) * val) ;
}
/* ------------------------------------------------------------------------ */
/*     Fonction rendant 1 si la valeur a et plus grande que b, sinon -1     */
/* ------------------------------------------------------------------------ */
double signe(int a,int b)
  { return( ( a > b ) ? 1 : -1 ) ;
}
/* ------------------------------------------------------------------------ */
/*        Fonction ralisant la rsolution d'un systme de 4 quations      */
/*                       linaires  4 inconnues                            */
/*            Utilisation de la descente et remonte de Gauss               */
/* ------------------------------------------------------------------------ */
void gauss (double *a,double *b,double *x)
  { int   i,j,k ;
    double  res ;

    for ( j = 0 ; j < 3 ; j++ ) {
      if ( a[j * 5] == 0 ) {
        for ( k = 1 ; a[(j + k) * 4 + j] == 0.0 ; k++ ) ;
        res = b[j + k] ;
        b[j + k] = b[j] ;
        b[j] = res ;
        for ( i = j ; i < 4 ; i ++ ) {
          res = a[(j + k) * 4 + i ] ;
          a[(j + k) * 4 + i] = a[j * 4 + i] ;
          a[j * 4 + i] = res ; } }
      for ( i = j + 1 ; i < 4 ; i++ ) {
        res = a[4 * i + j] / a[5 * j] ;
        b[i] = b[i] - res * b[j] ;
        for ( k = 0 ; k < 4 ; k++ )
          a[4 * i + k] -= (double) res * a[j * 4 + k] ; } }
    x[3] = b[3] / a[15] ;
    for ( i = 2 ; i >= 0 ; i-- ) {
      res = b[i] ;
      for ( j = 3 ; j > i ; j-- )
        res -= (x[j] * a[i * 4 + j]) ;
      x[i] = res / a[5 * i] ; }
}
/* ------------------------------------------------------------------------ */
/*             Fonction calculant les composantes a, b, c et d              */
/*     d'une portion de courbe de rsolution du lissage d'une feuille       */
/* ------------------------------------------------------------------------ */
void calcul_trinome(double *a,double *b,double *c,double *d,double di,double df,int xi,int yi,int xf,int yf)
  { double     x[16],t[4],y[4] ;

    x[1] = (double) xi * xi ;
    x[5] = (double) xf * xf ;
    x[0] = xi * x[1] ;
    x[4] = xf * x[5] ;
    x[8] = 3 * x[1] ;
    x[12] = 3 * x[5] ;
    x[9] = (double) 2 * xi ;
    x[13] = (double) 2 * xf ;
    x[2] = xi ;
    x[6] = xf ;
    x[7] = x[3] = x[14] = x[10] = 1 ;
    x[11] = 0 ;
    x[15] = 0 ;
    y[0] = yi ;
    y[1] = yf ;
    y[2] = di ;
    y[3] = df ;
    gauss(x,y,t) ;
    *a = t[0] ;
    *b = t[1] ;
    *c = t[2] ;
    *d = t[3] ;
}
/* ----------------------------------------------------------------------- */
double distance_sommets(double xi,double yi,double xf,double yf)
  { double    xx,yy ;

    xx = xf - xi ;
    yy = yf - yi ;
    return(sqrt(xx*xx+yy*yy)) ;
}
/* ----------------------------------------------------------------------- */
double distance_segment(double x,double y,double xi,double yi,double xf,double yf)
  { double    a,xx,yy ;

    if ( (yf != yi) || (xf != xi) ) {
      a = atan2(yf-yi,xf-xi) ;
      xf -= xi ;
      yf -= yi ;
      x -= xi ;
      y -= yi ;
      xx = x * cos(-a) - y * sin(-a) ;
      yy = x * sin(-a) + y * cos(-a) ;
      xi = xf * cos(-a) - yf * sin(-a) ;
      if ( xx < 0 )
        return(sqrt(xx*xx+yy*yy)) ;
        else
        if ( xx > xi )
          return(sqrt((xx-xi)*(xx-xi)+yy*yy)) ;
          else
          return(fabs(yy)) ; }
      else
      return(sqrt((x-xi)*(x-xi)+(y-yi)*(y-yi))) ;
}
/* ----------------------------------------------------------------------- */
double degre_to_radian(double v)
  { return(v * M_PI / 180) ;
}
/* ----------------------------------------------------------------------- */
double radian_to_degre(double v)
  { return(v * 180 / M_PI) ;
}
/* ----------------------------------------------------------------------- */
unsigned char point_inclu_triangle(struct coord_2D *p,struct coord_2D *c1,struct coord_2D *c2,struct coord_2D *c3)
  { struct coord_2D                  v1,v2,vv ;
    double                           disc,a,b ;

    if ( min_double(min_double(c1->x,c2->x),c3->x) <= p->x )
      if ( min_double(min_double(c1->y,c2->y),c3->y) <= p->y )
        if ( max_double(max_double(c1->y,c2->y),c3->y) >= p->y )
          if( max_double(max_double(c1->x,c2->x),c3->x) >= p->x ) {
            v2.x = c2->x - c1->x ;
            v2.y = c2->y - c1->y ;
            v1.x = c3->x - c1->x ;
            v1.y = c3->y - c1->y ;
            disc = v2.y*v1.x - v2.x*v1.y ;
            if ( fabs(disc) > 0.000000001 ) {
              vv.x = p->x - c1->x ;
              vv.y = p->y - c1->y ;
              b = (v1.x * vv.y - v1.y * vv.x) / disc ;
              if ( b > 0 ) {
                a = (v2.y * vv.x - v2.x * vv.y) / disc ;
                if ( ( a > 0 ) && ( a+b < 1 ) )
                  return(0x01) ; } } }
    return(0x00) ;
}

/* ************************************************************************* */
