#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "mathmath.h"
#include "coordonn.h"
#include "transfor.h"

/* *********************************************************************** */
/*           Module de gestion des transformations gomtriques            */
/* *********************************************************************** */

void affichageTrans(union transformation *t)
  { for ( int i = 0 ; i < 4 ; i++ ) {
      for ( int j = 0 ; j < 4 ; j++ )
        printf("%10.4f ",t->ttt[i][j]) ;
      printf("\n") ; }
    printf("\n") ;
}
/* ----------------------------------------------------------------------- */
void trans_identite(union transformation *t)
  { int                        i ;

    for ( i = 0 ; i < 16 ; i++ )
      t->tt[i] = 0.0 ;
    for ( i = 0 ; i < 4 ; i++ )
      t->ttt[i][i] = 1.0 ;
}
/* ----------------------------------------------------------------------- */
union transformation *nouvelle_trans(void)
  { return((union transformation *) calloc(16,sizeof(double))) ;
}
/* ----------------------------------------------------------------------- */
union transformation *nouvelle_trans_identite(void)
  { union transformation      *t ;

    t = nouvelle_trans() ;
    trans_identite(t) ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
void liberation_trans(union transformation *t)
  { free(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_rotation(double a,double x,double y,double z)
  { union transformation *t = nouvelle_trans_identite() ;
    a = a * M_PI / 180.0 ;
    double ca = cos(a) ;
    double sa = sin(a) ;
    double mca = 1 - ca ;
    double x2 = x*x ;
    double y2 = y*y ;
    double z2 = z*z ;
    t->to[0].co.x = x2+ca*(1-x2) ;
    t->to[0].co.y = x*y*mca-z*sa ;
    t->to[0].co.z = x*z*mca+y*sa ;
    t->to[1].co.x = x*y*mca+z*sa ;
    t->to[1].co.y = y2+ca*(1-y2) ;
    t->to[1].co.z = y*z*mca-x*sa ;
    t->to[2].co.x = x*z*mca-y*sa ;
    t->to[2].co.y = y*z*mca+x*sa ;
    t->to[2].co.z = z2+ca*(1-z2) ;
    affichageTrans(t) ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_scale(double x,double y,double z)
  { union transformation        *t ;

    t = nouvelle_trans_identite() ;
    t->to[0].co.x = x ;
    t->to[1].co.y = y ;
    t->to[2].co.z = z ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_zoom(double r)
  { union transformation        *t ;

    t = nouvelle_trans_identite() ;
    t->to[0].co.x = r ;
    t->to[1].co.y = r ;
    t->to[2].co.z = r ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_affinite_z(double r)
  { union transformation        *t ;

    t = nouvelle_trans_identite() ;
    t->to[2].co.z = r ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_affinite_y(double r)
  { union transformation        *t ;

    t = nouvelle_trans_identite() ;
    t->to[1].co.y = r ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_affinite_x(double r)
  { union transformation        *t ;

    t = nouvelle_trans_identite() ;
    t->to[0].co.x = r ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_affinite(struct affinite *af)
  { union transformation        *t ;

    t = nouvelle_trans_identite() ;
    t->to[0].co.x = af->ax ;
    t->to[1].co.y = af->ay ;
    t->to[2].co.z = af->az ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_rotation_x(double alpha)
  { union transformation        *t ;

    t = nouvelle_trans() ;
    t->to[0].co.x = 1.0 ;
    t->to[3].co.t = 1.0 ;
    t->to[1].co.y = t->to[2].co.z = cos(alpha) ;
    t->to[1].co.z = -sin(alpha) ;
    t->to[2].co.y = -t->to[1].co.z ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_rotation_y(double alpha)
  { union transformation        *t ;

    t = nouvelle_trans() ;
    t->to[1].co.y = 1.0 ;
    t->to[3].co.t = 1.0 ;
    t->to[2].co.z = t->to[0].co.x = cos(alpha) ;
    t->to[2].co.x = -sin(alpha) ;
    t->to[0].co.z = -t->to[2].co.x ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_rotation_z(double alpha)
  { union transformation        *t ;

    t = nouvelle_trans() ;
    t->to[2].co.z = 1.0 ;
    t->to[3].co.t = 1.0 ;
    t->to[1].co.y = t->to[0].co.x = cos(alpha) ;
    t->to[0].co.y = -sin(alpha) ;
    t->to[1].co.x = -t->to[0].co.y ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *transRotationx(double alpha)
  { union transformation        *t ;

    t = nouvelle_trans() ;
    t->to[0].co.x = 1.0 ;
    t->to[3].co.t = 1.0 ;
    t->to[1].co.y = t->to[2].co.z = cos(alpha) ;
    t->to[1].co.z = -sin(alpha) ;
    t->to[2].co.y = -t->to[1].co.z ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *transRotationy(double alpha)
  { union transformation        *t ;

    t = nouvelle_trans() ;
    t->to[1].co.y = 1.0 ;
    t->to[3].co.t = 1.0 ;
    t->to[2].co.z = t->to[0].co.x = cos(alpha) ;
    t->to[2].co.x = sin(alpha) ;
    t->to[0].co.z = -t->to[2].co.x ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *transRotationz(double alpha)
  { union transformation        *t ;

    t = nouvelle_trans() ;
    t->to[2].co.z = 1.0 ;
    t->to[3].co.t = 1.0 ;
    t->to[1].co.y = t->to[0].co.x = cos(alpha) ;
    t->to[0].co.y = -sin(alpha) ;
    t->to[1].co.x = -t->to[0].co.y ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_translation(double tx,double ty,double tz)
  { union transformation        *t ;

    t = nouvelle_trans_identite() ;
    t->to[0].co.t = tx ;
    t->to[1].co.t = ty ;
    t->to[2].co.t = tz ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
void multiplicationTrans(union transformation *a,union transformation *b,union transformation *r)
  { int                         i,j,k ;
    union transformation        *t = nouvelle_trans() ;
    for ( i = 0 ; i < 4 ; i++ )
      for ( j = 0 ; j < 4 ; j++ )
        for ( t->ttt[i][j] = 0,k = 0 ; k < 4 ; k++ )
          t->ttt[i][j] += a->ttt[i][k] * b->ttt[k][j] ;
    *r = *t ;
    liberation_trans(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *multiplication_trans(union transformation *a,union transformation *b)
  { union transformation        *t ;
    int                         i,j,k ;

    t = nouvelle_trans() ;
    for ( i = 0 ; i < 4 ; i++ )
      for ( j = 0 ; j < 4 ; j++ )
        for ( t->ttt[i][j] = 0,k = 0 ; k < 4 ; k++ )
          t->ttt[i][j] += a->ttt[i][k] * b->ttt[k][j] ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
unsigned char egalite_trans(union transformation *t1,union transformation *t2)
  { int                            i ;

    for ( i = 0 ; i < 16 ; i++ )
      if ( fabs(t1->tt[i]-t2->tt[i]) > 0.000001 )
        return(0x00) ;
    return(0x01) ;
}
/* ----------------------------------------------------------------------- */
void rotation_equivalente_2_rotation(struct orientation *o1,struct orientation *o2,struct orientation *o)
  { union transformation       *t1,*t2,*t,*to ;
    double                     rx[2],ry[2],rz[2],i,j,k ;

    t1=trans_orientation(o1) ;
    t2=trans_orientation(o2) ;
    t = multiplication_trans(t1,t2) ;
    liberation_trans(t1) ;
    liberation_trans(t2) ;
    rz[0] = asin(t->tt[4]) ;
    rz[1] = M_PI - rz[0] ;
    rx[0] = acos(t->tt[5]/cos(rz[0])) ;
    rx[1] = -rx[0] ;
    ry[0] = acos(t->tt[0]/cos(rz[0])) ;
    ry[1] = -ry[0] ;
    for ( i = 0 ; i < 2 ; i++ )
      for ( j = 0 ; j < 2 ; j++ )
        for ( k = 0 ; k < 2 ; k++ ) {
          o->rx = rx[(int) i] ;
          o->ry = ry[(int) j] ;
          o->rz = rz[(int) k] ;
          to=trans_orientation(o) ;
          if ( egalite_trans(to,t) )
            i = j = k = 2 ;
          liberation_trans(to) ; }
    liberation_trans(t) ;
}
/* ----------------------------------------------------------------------- */
void trans_orient_directe_et_inverse(struct orientation *ori,union transformation **t,union transformation **ti)
  { double                      cx,sx,cy,sy,cz,sz,cysz,sysz ;

    *t = nouvelle_trans_identite() ;
    *ti = nouvelle_trans_identite() ;
    cx = cos(ori->rx) ;
    sx = sin(ori->rx) ;
    cy = cos(ori->ry) ;
    sy = sin(ori->ry) ;
    cz = cos(ori->rz) ;
    sz = sin(ori->rz) ;
    (*t)->tt[0] = cy * cz ;
    (*t)->tt[1] = -(cysz = cy * sz) * cx + sy * sx ;
    (*t)->tt[2] = cysz * sx + sy * cx ;
    (*t)->tt[4] = sz ;
    (*t)->tt[5] = cz * cx ;
    (*t)->tt[6] = -cz * sx ;
    (*t)->tt[8] = -sy * cz ;
    (*t)->tt[9] = (sysz = sy * sz) * cx + sx * cy ;
    (*t)->tt[10] = -sysz * sx + cx * cy ;
    (*ti)->tt[0] = cy * cz ;
    (*ti)->tt[4] = -cysz * cx + sy * sx ;
    (*ti)->tt[8] = cysz * sx + sy * cx ;
    (*ti)->tt[1] = sz ;
    (*ti)->tt[5] = cz * cx ;
    (*ti)->tt[9] = -cz * sx ;
    (*ti)->tt[2] = -sy * cz ;
    (*ti)->tt[6] = sysz * cx + sx * cy ;
    (*ti)->tt[10] = -sysz * sx + cx * cy ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_orientation(struct orientation *ori)
  { union transformation        *t ;
    double                      cx,sx,cy,sy,cz,sz,cysz,sysz ;

    t = nouvelle_trans_identite() ;
    cx = cos(ori->rx) ;
    sx = sin(ori->rx) ;
    cy = cos(ori->ry) ;
    sy = sin(ori->ry) ;
    cz = cos(ori->rz) ;
    sz = sin(ori->rz) ;
    t->tt[0] = cy * cz ;
    t->tt[1] = -(cysz = cy * sz) * cx + sy * sx ;
    t->tt[2] = cysz * sx + sy * cx ;
    t->tt[4] = sz ;
    t->tt[5] = cz * cx ;
    t->tt[6] = -cz * sx ;
    t->tt[8] = -sy * cz ;
    t->tt[9] = (sysz = sy * sz) * cx + sx * cy ;
    t->tt[10] = -sysz * sx + cx * cy ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_orientation_inverse(struct orientation *ori)
  { union transformation        *t ;
    double                      cx,sx,cy,sy,cz,sz,cysz,sysz ;

    t = nouvelle_trans_identite() ;
    cx = cos(ori->rx) ;
    sx = sin(ori->rx) ;
    cy = cos(ori->ry) ;
    sy = sin(ori->ry) ;
    cz = cos(ori->rz) ;
    sz = sin(ori->rz) ;
    t->tt[0] = cy * cz ;
    t->tt[4] = -(cysz = cy * sz) * cx + sy * sx ;
    t->tt[8] = cysz * sx + sy * cx ;
    t->tt[1] = sz ;
    t->tt[5] = cz * cx ;
    t->tt[9] = -cz * sx ;
    t->tt[2] = -sy * cz ;
    t->tt[6] = (sysz = sy * sz) * cx + sx * cy ;
    t->tt[10] = -sysz * sx + cx * cy ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
void multiplication_trans_vect(union transformation *a,union coord_3D_homogenes *c)
  { union coord_3D_homogenes    t ;
    int                         i,j ;

    for ( i = 0 ; i < 3 ; i++ )
      for ( t.ct[i] = 0.,j = 0 ; j < 4 ; j++ )
        t.ct[i] += a->ttt[i][j] * c->ct[j] ;
    for ( i = 0 ; i < 3 ; i++ )
      c->ct[i] = t.ct[i] ;
    c->co.t = 1.0 ;
}
/* ----------------------------------------------------------------------- */
void multiplication_trans_dir(union transformation *a,union coord_3D_homogenes *c)
  { union coord_3D_homogenes    t ;
    int                         i,j ;

    for ( i = 0 ; i < 3 ; i++ )
      for ( t.ct[i] = 0.,j = 0 ; j < 3 ; j++ )
        t.ct[i] += a->ttt[i][j] * c->ct[j] ;
    for ( i = 0 ; i < 3 ; i++ )
      c->ct[i] = t.ct[i] ;
    c->co.t = 1.0 ;
}
/* ----------------------------------------------------------------------- */
void mult_trans_vect(union transformation *a,union coord_3D_hmg *c)
  { union coord_3D_hmg          t ;
    int                         i,j ;

    for ( i = 0 ; i < 3 ; i++ )
      for ( t.ct[i] = 0.F,j = 0 ; j < 4 ; j++ )
        t.ct[i] +=(float) a->ttt[i][j] * c->ct[j] ;
    for ( i = 0 ; i < 3 ; i++ )
      c->ct[i] = t.ct[i] ;
    c->co.t = 1.0F ;
}
/* ----------------------------------------------------------------------- */
void multiplication_trans_coordonnees(union transformation *a,union coord_3D_homogenes *c,int n)
  { union coord_3D_homogenes    t ;
    int                         i,j,k ;

    for ( k = 0 ; k < n ; k++ ) {
      for ( i = 0 ; i < 3 ; i++ )
        for ( t.ct[i] = 0.,j = 0 ; j < 4 ; j++ )
          t.ct[i] += a->ttt[i][j] * c[k].ct[j] ;
      for ( i = 0 ; i < 3 ; i++ )
        c[k].ct[i] = t.ct[i] ;
      c[k].co.t = 1.0 ; }
}
/* ----------------------------------------------------------------------- */
void mult_trans_coordonnees(union transformation *a,union coord_3D_hmg *c,int n)
  { union coord_3D_hmg          t ;
    int                         i,j,k ;

    for ( k = 0 ; k < n ; k++ ) {
      for ( i = 0 ; i < 3 ; i++ )
        for ( t.ct[i] = 0.F,j = 0 ; j < 4 ; j++ )
          t.ct[i] +=(float) a->ttt[i][j] * c[k].ct[j] ;
      for ( i = 0 ; i < 3 ; i++ )
        c[k].ct[i] = t.ct[i] ;
      c[k].co.t = 1.0F ; }
}
/* ----------------------------------------------------------------------- */
void init_coord_homogenes(union coord_3D_homogenes *c)
  { c->ct[0] = c->ct[1] = c->ct[2] = 0.0 ;
    c->ct[3] = 1.0 ;
}
/* ----------------------------------------------------------------------- */
union transformation *determine_trans_point_de_vue(struct point_de_vue *pv )
  { union transformation         *tr,*tint,*t ;

    tr = trans_translation(-pv->pv.co.x,-pv->pv.co.y,-pv->pv.co.z) ;
    tint = trans_orientation(&pv->pvor) ;
    t = multiplication_trans(tint,tr) ;
    liberation_trans(tint) ;
    liberation_trans(tr) ;
    return(t) ;
}
/* ----------------------------------------------------------------------- */
union transformation *trans_point_de_vue_inverse(struct point_de_vue *pv )
  { union transformation         *tr,*tint,*t ;

    tr = trans_translation(pv->pv.co.x,pv->pv.co.y,pv->pv.co.z) ;
    tint = trans_orientation_inverse(&pv->pvor) ;
    t = multiplication_trans(tr,tint) ;
    liberation_trans(tint) ;
    liberation_trans(tr) ;
    return(t) ;
}

/* *********************************************************************** */
