#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "trigonom.h"
#include "coordonn.h"
#include "troisdim.h"
#include "transfor.h"

void    rot_x(double *mat,int dx,int m) ;
void    rot_y(double *mat,int dy,int m) ;
void    rot_z(double *mat,int dz,int m) ;

/* ************************************************************************ */
/*                            MODULE DE GESTION 3D                          */
/* ************************************************************************ */

/* ------------------------------------------------------------------------ */
/*      Fonction d'initialisation d'une matrice carre de dimension n       */
/*               possdant toutes ses composantes gales  0                */
/*              sauf sur la diagonale o elles sont gales  1              */
/* ------------------------------------------------------------------------ */
double *init_mat(int n)
  { double     *mat ;

    mat =(double *) calloc(n*n,sizeof(double)) ;
    mat_invariant(mat,n) ;
    return(mat) ;
}
/* ------------------------------------------------------------------------ */
void mat_invariant(double *mat,int n)
  { int       i ;


    for ( i = 0 ; i < n*n ; i++ )
      mat[i] = 0.0 ;
    for ( i = 0 ; i < n ; i++ )
      mat[i*(n+1)] = 1.0 ;
}
/* ------------------------------------------------------------------------ */
/*         Calcul la matrice de rotation par rapport  l'axe des x          */
/*                    pour une rotation de dx radians                       */
/* ------------------------------------------------------------------------ */
void rot_x_reel(double *mat,double dx,int n)
  { mat_invariant(mat,n) ;
    mat[n+1] =  cos(dx) ;
    mat[n+2] = -sin(dx) ;
    mat[2*n+1] =  sin(dx) ;
    mat[2*n+2] =  cos(dx) ;
}
/* ------------------------------------------------------------------------ */
/*         Calcul la matrice de rotation par rapport  l'axe des y          */
/*                    pour une rotation de dy radians                       */
/* ------------------------------------------------------------------------ */
void rot_y_reel(double *mat,double dy,int n)
  { mat_invariant(mat,n) ;
    mat[0] =  cos(dy) ;
    mat[2] =  sin(dy) ;
    mat[2*n] = -sin(dy) ;
    mat[2*n+2] =  cos(dy) ;
}
/* ------------------------------------------------------------------------ */
/*         Calcul la matrice de rotation par rapport  l'axe des z          */
/*                    pour une rotation de dz radians                       */
/* ------------------------------------------------------------------------ */
void rot_z_reel(double *mat,double dz,int n)
  { mat_invariant(mat,n) ;
    mat[0] = cos(dz) ;
    mat[1] = -sin(dz) ;
    mat[n] = sin(dz) ;
    mat[n+1] = cos(dz) ;
}
/* ------------------------------------------------------------------------ */
/*       Ralise une rotation sur un tableau de nombres de type double      */
/*                        angles donns en radians                          */
/* ------------------------------------------------------------------------ */
void rotation_reel(double *mat,double dx,double dy,double dz,int n,int m)
  { double             *temp1,*temp2 ;

    temp1 = init_mat(m) ;
    temp2 = init_mat(m) ;
    rot_x_reel(temp1,dx,m) ;
    rot_y_reel(temp2,dy,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    rot_z_reel(temp2,dz,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    prodmat_reel((double *) mat,temp1,(double *) mat,n,m) ;
    free(temp1) ;
    free(temp2) ;
}
/* ------------------------------------------------------------------------ */
/*       Ralise une rotation sur un tableau de nombres de type double      */
/*                        angles donns en radians                          */
/* ------------------------------------------------------------------------ */
void rotation_reel_entier(double *mat,int dx,int dy,int dz,int n,int m)
  { double             *temp1,*temp2 ;

    temp1 = init_mat(m) ;
    temp2 = init_mat(m) ;
    rot_x(temp1,dx,m) ;
    rot_y(temp2,dy,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    rot_z(temp2,dz,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    prodmat_reel(mat,temp1,mat,n,m) ;
    free(temp1) ;
    free(temp2) ;
}
/* ------------------------------------------------------------------------ */
/*       Ralise une rotation sur un tableau de nombres de type double      */
/*                        angles donns en radians                          */
/* ------------------------------------------------------------------------ */
void rotation_reel_court_entier(float *mat,int dx,int dy,int dz,int n,int m)
  { double             *temp1,*temp2 ;

    temp1 = init_mat(m) ;
    temp2 = init_mat(m) ;
    rot_x(temp1,dx,m) ;
    rot_y(temp2,dy,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    rot_z(temp2,dz,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    prodmat_reel_court(mat,temp1,mat,n,m) ;
    free(temp1) ;
    free(temp2) ;
}
/* ------------------------------------------------------------------------ */
/*   Ralise le produit matriciel du tableau de coordonnes 3D entires A   */
/*                             de dimension n                               */
/*     par la matrice 3x3 de rels B pour obtenir le tableau d'entiers C    */
/* ------------------------------------------------------------------------ */
void prodmat (int *a,double *b,int *c,int n,int m)
  { int         j,k,i ;
    double      somme,temp[4] ;

    for ( i = 0 ; i < n ; i++ ) {
      for ( j = 0 ; j < m ; j++ ) {
        somme = 0.0 ;
        for ( k = 0 ; k < m ; k++ )
          somme += a[i*m+k] * b[k*m+j] ;
        temp[j] = somme ; }
      for ( j = 0 ; j < m ; j++ )
        c[i*m+j] =(int) temp[j] ; }
}
/* ------------------------------------------------------------------------ */
/*   Ralise le produit matriciel du tableau de coordonnes 3D rellees A   */
/*                             de dimension n                               */
/*     par la matrice 3x3 de rels B pour obtenir le tableau de rels C     */
/* ------------------------------------------------------------------------ */
void prodmat_reel (double *a,double *b,double *c,int n,int m)
  { int         i,j,k ;
    double      somme,temp[4] ;

    for ( i = 0 ; i < n ; i++ ) {
      for ( j = 0 ; j < m ; j++ ) {
        somme = 0.0 ;
        for ( k = 0 ; k < m ; k++ )
          somme += a[i*m+k] * b[k*m+j] ;
        temp[j] = somme ; }
      for ( j = 0 ; j < m ; j++ )
        c[i*m+j] = temp[j] ; }
}
/* ------------------------------------------------------------------------ */
void prodmat_reel_vecteur_3(double *a,double *b)
  { int         i,k ;
    double      somme,temp[3] ;

    for ( i = 0 ; i < 9 ; i += 4 ) {
      somme = 0.0 ;
      for ( k = 0 ; k < 3 ; k++ )
        somme += a[i+k] * b[k] ;
      temp[i>>2] = somme + a[i+3]  ; }
    memcpy(b,temp,24) ;
}
/* ------------------------------------------------------------------------ */
void prodmat_reel_vecteur(double *a,double *b,double *c,int n)
  { int         i,k ;
    double      somme,temp[4] ;

    for ( i = 0 ; i < n ; i++ ) {
      somme = 0.0 ;
      for ( k = 0 ; k < n ; k++ )
        somme += a[i*n+k] * b[k] ;
      temp[i] = somme ; }
    for ( i = 0 ; i < n ; i++ )
      c[i] = temp[i] ;
}
/* ------------------------------------------------------------------------ */
/*   Ralise le produit matriciel du tableau de coordonnes 3D rellees A   */
/*                             de dimension n                               */
/*     par la matrice 3x3 de rels B pour obtenir le tableau de rels C     */
/* ------------------------------------------------------------------------ */
void prodmat_reel_court (float *a,double *b,float *c,int n,int m)
  { int         i,j,k ;
    double      somme,temp[4] ;

    for ( i = 0 ; i < n ; i++ ) {
      for ( j = 0 ; j < m ; j++ ) {
        somme = 0.0 ;
        for ( k = 0 ; k < m ; k++ )
          somme += a[i*m+k] * b[k*m+j] ;
        temp[j] = somme ; }
      for ( j = 0 ; j < m ; j++ )
        c[i*m+j] =(float) temp[j] ; }
}
/* ------------------------------------------------------------------------ */
/*         Calcul la matrice de rotation par rapport  l'axe des x          */
/*                    pour une rotation de dx degrs                        */
/* ------------------------------------------------------------------------ */
void rot_x(double *mat,int dx,int n)
  { mat_invariant(mat,n) ;
    mat[n+1] = r_cs(dx) ;
    mat[n+2] = -r_sn(dx) ;
    mat[2*n+1] = r_sn(dx) ;
    mat[2*n+2] = r_cs(dx) ;
}
/* ------------------------------------------------------------------------ */
/*         Calcul la matrice de rotation par rapport  l'axe des y          */
/*                    pour une rotation de dy degrs                        */
/* ------------------------------------------------------------------------ */
void rot_y(double *mat,int dy,int n)
  { mat_invariant(mat,n) ;
    mat[0] = r_cs(dy) ;
    mat[2] = r_sn(dy) ;
    mat[2*n] = -r_sn(dy) ;
    mat[2*n+2] = r_cs(dy) ;
}
/* ------------------------------------------------------------------------ */
/*         Calcul la matrice de rotation par rapport  l'axe des z          */
/*                    pour une rotation de dz degrs                        */
/* ------------------------------------------------------------------------ */
void rot_z(double *mat,int dz,int n)
  { mat_invariant(mat,n) ;
    mat[0] = r_cs(dz) ;
    mat[1] = -r_sn(dz) ;
    mat[n] = r_sn(dz) ;
    mat[n+1] = r_cs(dz) ;
}
/* ------------------------------------------------------------------------ */
/*       Ralise une rotation sur un tableau de nombres de type entier      */
/*                        angles donns en degrs                           */
/* ------------------------------------------------------------------------ */
void rotation_entier(int *mat,int dx,int dy,int dz,int n,int m)
  { double      *temp1,*temp2 ;

    temp1 = init_mat(m) ;
    temp2 = init_mat(m) ;
    rot_x(temp1,dx,m) ;
    rot_y(temp2,dy,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    rot_z(temp2,dz,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    prodmat(mat,temp1,mat,n,m) ;
    free(temp1) ;
    free(temp2) ;
}
/* ------------------------------------------------------------------------ */
/*        Rotation et zoom de valeurs identiques pour deux matrices         */
/* ------------------------------------------------------------------------ */
void rotation_zoom_2_matrices(int *mat1,int *mat2,int dx,int dy,int dz,double z,int n1,int n2,int m)
  { double          *temp1,*temp2,*temp ;

    temp1 = init_mat(m) ;
    temp2 = init_mat(m) ;
    rot_x(temp1,dx,m) ;
    rot_y(temp2,dy,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    rot_z(temp2,dz,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    temp = init_mat(m) ;
    temp[0] = z ;
    temp[m+1] = z ;
    temp[2*m+2] = z ;
    prodmat_reel(temp1,temp,temp1,m,m) ;
    prodmat(mat2,temp1,mat2,n2,m) ;
    prodmat(mat1,temp1,mat1,n1,m) ;
    free(temp1) ;
    free(temp2) ;
    free(temp) ;
}
/* ------------------------------------------------------------------------ */
double *rotation_zoom(int dx,int dy,int dz,double z,int m)
  { double          *temp1,*temp2,*temp ;

    temp1 = init_mat(m) ;
    temp2 = init_mat(m) ;
    rot_x(temp1,dx,m) ;
    rot_y(temp2,dy,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    rot_z(temp2,dz,m) ;
    prodmat_reel(temp1,temp2,temp1,m,m) ;
    temp = init_mat(m) ;
    temp[0] = z ;
    temp[m+1] = z ;
    temp[2*m+2] = z ;
    prodmat_reel(temp1,temp,temp1,m,m) ;
    free(temp) ;
    free(temp2) ;
    return(temp1) ;
}
/* ------------------------------------------------------------------------ */
void rotation_zoom_1_matrice(int *mat1,int dx,int dy,int dz,double z,int n1,int m)
  { double          *temp ;

    temp = rotation_zoom(dx,dy,dz,z,m) ;
    prodmat(mat1,temp,mat1,n1,m) ;
    free(temp) ;
}
/* ------------------------------------------------------------------------ */
void zoom(int *mat,double ex,double ey,double ez,int n,int m)
  { double          *temp ;

    temp = init_mat(m) ;
    temp[0] = ex ;
    temp[m+1] = ey ;
    temp[2*m+2] = ez ;
    prodmat(mat,temp,mat,n,m) ;
    free(temp) ;
}
/* ------------------------------------------------------------------------ */
void translation(double *mat,double tx,double ty,double tz,int n)
  { mat_invariant(mat,n) ;
    mat[n-1] = tx ;
    mat[2*n-1] = ty ;
    if ( n > 3 )
      mat[3*n-1] = tz ;
}
/* ------------------------------------------------------------------------ */
void multiplication_coord_4D(struct coord_4D *c,double f)
  { c->x *=(float) f ;
    c->y *=(float) f ;
    c->z *=(float) f ;
}
/* ------------------------------------------------------------------------ */
void multiplication_coord_3D(struct coord_3D_court *c,double f)
  { c->x *=(float) f ;
    c->y *=(float) f ;
    c->z *=(float) f ;
}
/* ------------------------------------------------------------------------ */
void m_translation(double *mat,struct coord_3D *t)
  { mat_invariant(mat,4) ;
    mat[3] = -t->x ;
    mat[7] = -t->y ;
    mat[11] = -t->z ;
}
/* ------------------------------------------------------------------------ */
double *calcul_transformation_placement(union coord_3D_homogenes *ce,struct orientation *ori,struct affinite *af,double taille)
  { double                *m,*mm,*maff,*mtr,*mrot ;
    struct coord_3D       c ;
    struct orientation    o ;
    struct affinite       a ;
    o.rx = -ori->rx ;
    o.ry = -ori->ry ;
    o.rz = -ori->rz ;
    a.ax = 1/af->ax ;
    a.ay = 1/af->ay ;
    a.az = 1/af->az ;
    c.x =(__TYPE__) ce->co.x ;
    c.y =(__TYPE__) ce->co.y ;
    c.z =(__TYPE__) ce->co.z ;
    m =(double *) calloc(16,sizeof(double)) ;
    mm =(double *) calloc(16,sizeof(double)) ;
    mtr =(double *) calloc(16,sizeof(double)) ;
    maff =(double *) trans_affinite(&a) ;
    maff[0] /= taille ;
    maff[5] /= taille ;
    maff[10] /= taille ;
    mrot =(double *) trans_orientation(&o) ;
    m_translation(mtr,&c) ;
    prodmat_reel(mrot,mtr,mm,4,4) ;
    prodmat_reel(maff,mm,m,4,4) ;
    free(mm) ;
    free(maff) ;
    free(mrot) ;
    free(mtr) ;
    return(m) ;
}
/* ------------------------------------------------------------------------ */
void produit_matrice(double *b,float *a,float *c)
  { int         j,k ;
    double      somme,temp[4] ;

    for ( j = 0 ; j < 4 ; j++ ) {
      somme = 0.0 ;
      for ( k = 0 ; k < 3 ; k++ )
        somme += b[4*j+k] * a[k] ;
      temp[j] = somme ; }
    for ( j = 0 ; j < 4 ; j++ )
      c[j] =(float) temp[j] ;
}

/* ************************************************************************ */
