/**
 * \file Dilatation.h
 * Implante un filtre de de dilatation en morphologie mathematique.
 * 
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */

#ifndef ____DILATATION____
#define ____DILATATION____

#include "Dilatation.h"
#include "Image.h"
#include "ElementStructurant.h"

/**
 * Filtre une image par dilatation.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param e l'elementStructurant de dilatation.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageDilatation(struct image *src,struct image *dst,struct elementStructurant *e);
  
/**
 * Filtre une image par dilatation sans allocation de memoire pour l'image resultat.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param deb le numero de la ligne de debut du filtrage.
 * @param fin le numero de la ligne de fin du filtrage.
 * @param e l'elementStructurant de dilatation.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageDilatation(struct image *src,struct image *dst,int deb,int fin,struct elementStructurant *e);

/**
 * Filtre une image par dilatation.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param e l'elementStructurant de dilatation.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageDilatation(int callage,struct image *src,struct image *dst,struct elementStructurant *e);
  
/**
 * Filtre une image par dilatation sans allocation de memoire pour l'image resultat.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param deb le numero de la ligne de debut du filtrage.
 * @param fin le numero de la ligne de fin du filtrage.
 * @param e l'elementStructurant de dilatation.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageDilatation(int callage,struct image *src,struct image *dst,int deb,int fin,struct elementStructurant *e);

#endif
