/**
 * \file Fermeture.h
 * Implante un filtre de fermeture en morphologie mathematique.
 * 
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */

#ifndef ____FERMETURE____
#define ____FERMETURE____

#include "Image.h"
#include "ElementStructurant.h"

/**
 * Filtre une image par fermeture.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param e l'elementStructurant de fermeture.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageFermeture(struct image *src,struct image *dst,struct elementStructurant *e);
 
/**
 * Filtre une image par fermeture.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param e l'elementStructurant de fermeture.
 * @param n le nombre d'iterations de la fermeture.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageFermeture(struct image *src,struct image *dst,struct elementStructurant *e,int n);

/**
 * Filtre une image par fermeture.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param e l'elementStructurant de fermeture.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageFermeture(int callage,struct image *src,struct image *dst,struct elementStructurant *e);
 
/**
 * Filtre une image par fermeture.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param e l'elementStructurant de fermeture.
 * @param n le nombre d'iterations de la fermeture.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageFermeture(int callage,struct image *src,struct image *dst,struct elementStructurant *e,int n);

#endif
