/**
 * \file FermetureElementComplet.cpp
 * Implante un filtre de fermeture par l'element structurant complet en morphologie mathematique.
 * 
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */

#include <stdio.h>
#include <stdlib.h>

#include "Fermeture.h"
#include "FermetureElementComplet.h"
#include "Image.h"
#include "ElementStructurant.h"

/**
 * Filtre une image par fermeture par l'element structurant complet.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageFermetureElementComplet(struct image *src,struct image *dst) {
  filtreImageFermeture(src,dst,&ELEMENT_STRUCTURANT_COMPLET);
}
 
/**
 * Filtre une image par fermeture par l'element structurant complet.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param n le nombre d'iterations de l'ouverture.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageFermetureElementComplet(struct image *src,struct image *dst,int n) {
  filtreImageFermeture(src,dst,&ELEMENT_STRUCTURANT_COMPLET,n);
}

/**
 * Filtre une image par fermeture par l'element structurant complet.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageFermetureElementComplet(int callage,struct image *src,struct image *dst) {
  filtreImageFermeture(callage,src,dst,&ELEMENT_STRUCTURANT_COMPLET);
}
 
/**
 * Filtre une image par fermeture par l'element structurant complet.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param n le nombre d'iterations de l'ouverture.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageFermetureElementComplet(int callage,struct image *src,struct image *dst,int n) {
  filtreImageFermeture(callage,src,dst,&ELEMENT_STRUCTURANT_COMPLET,n);
}
