/**
 * \file OuvertureElementComplet.cpp
 * Implante un filtre d'ouverture par l'element structurant complet en morphologie mathematique.
 * 
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */

#include <stdio.h>
#include <stdlib.h>

#include "Ouverture.h"
#include "OuvertureElementComplet.h"
#include "Image.h"
#include "ElementStructurant.h"

/**
 * Filtre une image par ouverture par l'element structurant complet.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageOuvertureElementComplet(struct image *src,struct image *dst) {
  filtreImageOuverture(src,dst,&ELEMENT_STRUCTURANT_COMPLET);
}
 
/**
 * Filtre une image par ouverture par l'element structurant complet.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param n le nombre d'iterations de l'ouverture.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageOuvertureElementComplet(struct image *src,struct image *dst,int n) {
  filtreImageOuverture(src,dst,&ELEMENT_STRUCTURANT_COMPLET,n);
}

/**
 * Filtre une image par ouverture par l'element structurant complet.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageOuvertureElementComplet(int callage,struct image *src,struct image *dst) {
  filtreImageOuverture(callage,src,dst,&ELEMENT_STRUCTURANT_COMPLET);
}
 
/**
 * Filtre une image par ouverture par l'element structurant complet.
 *
 * @param src l'image a filtrer.
 * @param dst l'image resultat apres filtrage.
 * @param n le nombre d'iterations de l'ouverture.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 12/01/06
 */
void filtreImageOuvertureElementComplet(int callage,struct image *src,struct image *dst,int n) {
  filtreImageOuverture(callage,src,dst,&ELEMENT_STRUCTURANT_COMPLET,n);
}
