/* Un atome d'helium anime                      */
/*                                              */
/* Auteur: Nicolas JANEY                        */
/* nicolas.janey@univ-fcomte.fr                 */
/* Janvier 2017                                 */

#include <stdlib.h>
#include <stdio.h>

#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>

/* Variables et constantes globales             */

static const float blanc[] = { 1.0F,1.0F,1.0F,1.0F };
static const float jaune[] = { 1.0F,1.0F,0.0F,1.0F };
static const float rouge[] = { 1.0F,0.0F,0.0F,1.0F };
static const float vert[] = { 0.0F,1.0F,0.0F,1.0F };
static const float bleu[] = { 0.0F,0.0F,1.0F,1.0F };

/* Fonction d'initialisation des parametres     */
/* OpenGL ne changeant pas au cours de la vie   */
/* du programme                                 */

static void init(void) {
  const GLfloat shininess[] = { 50.0 };
  glMaterialfv(GL_FRONT,GL_SPECULAR,blanc);
  glMaterialfv(GL_FRONT,GL_SHININESS,shininess);
  glLightfv(GL_LIGHT0,GL_DIFFUSE,rouge);
  glLightfv(GL_LIGHT1,GL_DIFFUSE,jaune);
  glLightfv(GL_LIGHT2,GL_DIFFUSE,bleu);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glEnable(GL_LIGHT2);
  glDepthFunc(GL_LESS);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
  glEnable(GL_AUTO_NORMAL);
}

//////////////////////////////////////////////////

/* Variables globales                           */

static float r0 = 0.0F;
static float rx = 0.0F;
static int affichageTrajectoire = 0;
static int rotationElectrons = 1;
static int mode = 1;
static float positionAngulaire = 0.0F;
static float delta = 3.0F;

/* Scene dessinee                               */

static void electron(float inclinaison,float positionAngulaire,int trajectoire) {
  glPushMatrix();
  glRotatef(inclinaison,0.0F,0.0F,1.0F);
  if ( trajectoire ) {
    glPushMatrix();
    glRotatef(90.0F,1.0F,0.0F,0.0F);
    glutSolidTorus(0.05,7.0,24,90);
    glPopMatrix(); }
  glRotatef(positionAngulaire,1.0F,0.0F,0.0F);
  glTranslatef(0.0F,7.0F,0.0F);
  glutSolidSphere(0.5,36,36);
  glPopMatrix();
}

static void scene(void) {
  glPushMatrix();
  glPushMatrix();
  glTranslatef(0.0F,1.0F,0.0F);
  glutSolidSphere(1.0,36,36);
  glPopMatrix();
  glPushMatrix();
  glTranslatef(0.0F,-1.0F,0.0F);
  glutSolidSphere(1.0,36,36);
  glPopMatrix();
  glPushMatrix();
  glTranslatef(1.732F,0.0F,0.0F);
  glutSolidSphere(1.0,36,36);
  glPopMatrix();
  glPushMatrix();
  glTranslatef(-1.732F,0.0F,0.0F);
  glutSolidSphere(1.0,36,36);
  glPopMatrix();
  electron(45.0F,positionAngulaire,affichageTrajectoire);
  electron(-45.0F,positionAngulaire+180.0F,affichageTrajectoire);
  glPopMatrix();
}

/* Fonction executee lors d'un rafraichissement */
/* de la fenetre de dessin                      */

static void display(void) {
  glClearColor(0.5F,0.5F,0.5F,0.5F);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  const GLfloat light0_position[] = { 0.0,0.0,0.0,1.0 };
  const GLfloat light1_position[] = { -1.0,1.0,1.0,0.0 };
  const GLfloat light2_position[] = { 1.0,-1.0,1.0,0.0 };
  glLightfv(GL_LIGHT0,GL_POSITION,light0_position);
  glLightfv(GL_LIGHT1,GL_POSITION,light1_position);
  glLightfv(GL_LIGHT2,GL_POSITION,light2_position);
  glPushMatrix();
  glRotatef(rx,1.0F,0.0F,0.0F);
  glRotatef(r0,1.0F,1.0F,1.0F);
  glPolygonMode(GL_FRONT_AND_BACK,(mode) ? GL_FILL : GL_LINE);
  scene();
  glPopMatrix();
  glFlush();
  glutSwapBuffers();
  int error = glGetError();
  if ( error != GL_NO_ERROR )
    printf("Attention erreur %d\n",error);
}

/* Fonction executee lorsqu'aucun evenement     */
/* n'est en file d'attente                      */

static void idle(void) {
  r0 += 1.0F;
  if ( rotationElectrons )
    positionAngulaire += delta;
  glutPostRedisplay();
}

/* Fonction executee lors de l'appui            */
/* d'une touche alphanumerique du clavier       */

static void keyboard(unsigned char key,int x,int y) {
  switch (key) {
    case 43 :
      { delta += 0.5F; }
      break;
    case 45 :
      { delta -= 0.5F; }
      break;
    case 0x0D :
      { static int anim = 0;
        anim = !anim;
        glutIdleFunc(( anim ) ? idle : NULL); }
      break;
    case 0x20 :
      { affichageTrajectoire = !affichageTrajectoire;
        glutPostRedisplay(); }
      break;
    case 0x1B :
      exit(0);
      break; }
}

static void special(int code,int x,int y) {
  switch (code) {
    case GLUT_KEY_PAGE_DOWN :
      { rx += 1.0F ;
        glutPostRedisplay(); }
      break;
    case GLUT_KEY_PAGE_UP :
      { rx -= 1.0F ;
        glutPostRedisplay(); }
      break; }
}

static void mouse(int button,int state,int x,int y) {
  switch(button) {
    case GLUT_LEFT_BUTTON :
      if ( state == GLUT_UP )
        rotationElectrons = !rotationElectrons;
      glutPostRedisplay();
      break;
    case GLUT_RIGHT_BUTTON :
      if ( state == GLUT_UP )
        mode = !mode;
      glutPostRedisplay();
      break; }
}

//////////////////////////////////////////////////

/* Fonction executee lors d'un changement       */
/* de la taille de la fenetre OpenGL            */

static void reshape(int x,int y) {
  glViewport(0,0,x,y); 
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0F,(float) x/y,1.0,40.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0.0,0.0,20.0,0.0,0.0,0.0,0.0,1.0,0.0);
}

/* Fonction principale                          */

int main(int argc,char **argv) {
  glutInit(&argc,argv);
  glutInitDisplayMode(GLUT_RGBA|GLUT_DEPTH|GLUT_DOUBLE);
  glutInitWindowSize(300,300);
  glutInitWindowPosition(50,50);
  glutCreateWindow("Un atome d'helium");
  init();
  glutSpecialFunc(special);
  glutKeyboardFunc(keyboard);
  glutReshapeFunc(reshape);
  glutMouseFunc(mouse);
  //glutIdleFunc(idle);
  glutDisplayFunc(display);
  glutMainLoop();
  return(0);
}
