/* Stockage d'une lumiere spot              */
/*                                          */
/* Auteur: Nicolas JANEY                    */
/* nicolas.janey@univ-fcomte.fr             */
/* Novembre 2012                            */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "LumiereSpot.h"
#include "LumierePonctuelle.h"
#include "Position3D.h"
#include "Direction3D.h"
#include "Energie.h"
#include "Couleur.h"

/* Constructeurs                            */
LumiereSpot::LumiereSpot(void):LumierePonctuelle() {
  d = new Direction3D(0.0,0.0,-1.0);
  ouverture = 180.0;
}

LumiereSpot::LumiereSpot(LumiereSpot *ls):LumierePonctuelle(ls) {
  d = new Direction3D(ls->d);
  ouverture = ls->ouverture;
}

/* Destructeur                              */
LumiereSpot::~LumiereSpot(void) {
  delete(d);
}

//////////////////////////////////////////////

#ifndef M_PI
#define M_PI 3.14159
#endif

/* Methode de calcul de la quantite         */
/* d'energie diffusee en un point p         */
/* de normale n et de coefficient           */
/* de diffusion kd                          */
void LumiereSpot::energieDiffusee(Energie *e,Position3D *p,Direction3D *n,Couleur *kd) {
  if ( !on ) {
    e->r = 0.0;
    e->v = 0.0;
    e->b = 0.0;
    return; }
  if ( ouverture > 90.0 ) {
    LumierePonctuelle::energieDiffusee(e,p,n,kd);
    return; }
  Direction3D *dirm = new Direction3D(d);
  Direction3D *dm = new Direction3D(p,this->p);
  double dist = dm->normalisation();
  dirm->normalisation();
  double ps = -dirm->produitScalaire(dm);
  delete(dirm);
  if ( ps < cos(ouverture*M_PI/180.0) ) {
    e->r = 0.0;
    e->v = 0.0;
    e->b = 0.0;
    return; }
  Direction3D *nm = new Direction3D(n);
  nm->normalisation();
  ps = nm->produitScalaire(dm);
  delete(nm);
  delete(dm);
  if ( ps < 0.0 ) {
    e->r = 0.0;
    e->v = 0.0;
    e->b = 0.0; }
    else {
    e->r = intensite*c->r*ps*kd->r;
    e->v = intensite*c->v*ps*kd->v;
    e->b = intensite*c->b*ps*kd->b;
    if ( attenuation ) {
      e->r /= (dist*dist);
      e->v /= (dist*dist);
      e->b /= (dist*dist); } }
}

//////////////////////////////////////////////

/* Methode d'affichage texte                */
void LumiereSpot::print(void) {
  LumierePonctuelle::print();
  printf(" - ");
  d->print();
  printf(" - %lf",ouverture);
}
