/* Stockage d'une lumiere directionnelle    */
/*                                          */
/* Auteur: Nicolas JANEY                    */
/* nicolas.janey@univ-fcomte.fr             */
/* Mars 2014                                */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "LumiereDirectionnelle.h"
#include "Dir3D.h"
#include "Energie.h"
#include "Couleur.h"
#include "Scene.h"
#include "RayonLumineux.h"
#include "TG3D.h"

#define INFINI 1E+100

/* Constructeurs                            */
LumiereDirectionnelle::LumiereDirectionnelle(void):Lumiere() {
  d = new Dir3D(0.0,0.0,-1.0);
}

LumiereDirectionnelle::LumiereDirectionnelle(Dir3D *direction,double energie,Couleur *couleur):Lumiere(couleur,energie) {
  d = new Dir3D(direction);
}

LumiereDirectionnelle::LumiereDirectionnelle(LumiereDirectionnelle *ld):Lumiere(ld) {
  d = new Dir3D(ld->d);
}

/* Destructeur                              */
LumiereDirectionnelle::~LumiereDirectionnelle(void) {
  delete(d);
}

//////////////////////////////////////////////

/* Methode de calcul de la quantite         */
/* d'energie diffusee en un point p         */
/* de normale n et de coefficient           */
/* de diffusion kd                          */

void LumiereDirectionnelle::energieDiffusee(Energie *e,Pos3D *p,Dir3D *n,Couleur *kd) {
  if ( !on ) {
    e->r = 0.0;
    e->v = 0.0;
    e->b = 0.0;
    return; }
  Dir3D *nm = new Dir3D(n);
  Dir3D *dm = new Dir3D(d);
  nm->normalisation();
  dm->normalisation();
  double ps = nm->produitScalaire(dm);
  delete(nm);
  delete(dm);
  if ( ps < 0.0 ) {
    e->r = 0.0;
    e->v = 0.0;
    e->b = 0.0; }
    else {
    e->r = intensite*c->r*ps*kd->r;
    e->v = intensite*c->v*ps*kd->v;
    e->b = intensite*c->b*ps*kd->b; }
}

//////////////////////////////////////////////

int LumiereDirectionnelle::eclaire(Pos3D *p,Scene *scn,double dist) {
  RayonLumineux *rlo = new RayonLumineux(p,d);
  rlo->direction->normalisation();
  int res = !scn->intersection(rlo);
  delete(rlo);
  return(res);
}

/* Methode d'affichage texte                */
void LumiereDirectionnelle::print(void) {
  Lumiere::print();
  printf(" - ");
  d->print();
}

double LumiereDirectionnelle::distance(Pos3D *p) {
  return(INFINI);
}

void LumiereDirectionnelle::transforme(TG3D *m) {
  d->transformation(m);
}

void LumiereDirectionnelle::modeliseOpenGL(int num) {
  float dir[4] = { d->x,d->y,d->z,0.0F };
  glLightfv(num,GL_POSITION,dir);
}

LumiereDirectionnelle *LumiereDirectionnelle::clone(void) {
  return(new LumiereDirectionnelle(this));
}
