/* Stockage d'une lumiere ponctuelle        */
/*                                          */
/* Auteur: Nicolas JANEY                    */
/* nicolas.janey@univ-fcomte.fr             */
/* Mars 2014                                */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "LumierePonctuelle.h"
#include "Pos3D.h"
#include "Dir3D.h"
#include "Energie.h"
#include "Couleur.h"
#include "RayonLumineux.h"
#include "Scene.h"
#include "TG3D.h"

/* Constructeurs                            */
LumierePonctuelle::LumierePonctuelle(void):Lumiere() {
  p = new Pos3D();
  attenuation = false;
}

LumierePonctuelle::LumierePonctuelle(Pos3D *position,double energie,Couleur *couleur):Lumiere(couleur,energie) {
  p = new Pos3D(position);
  attenuation = true;
}

LumierePonctuelle::LumierePonctuelle(LumierePonctuelle *lp):Lumiere(lp) {
  p = new Pos3D(lp->p);
  attenuation = lp->attenuation;
}

/* Destructeur                              */
LumierePonctuelle::~LumierePonctuelle(void) {
  delete(p);
}

//////////////////////////////////////////////

/* Methode de calcul de la quantite         */
/* d'energie diffusee en un point p         */
/* de normale n et de coefficient           */
/* de diffusion kd                          */

void LumierePonctuelle::energieDiffusee(Energie *e,Pos3D *p,Dir3D *n,Couleur *kd) {
  if ( !on ) {
    e->r = 0.0;
    e->v = 0.0;
    e->b = 0.0;
    return; }
  Dir3D *nm = new Dir3D(n);
  Dir3D *dm = new Dir3D(p,this->p);
  nm->normalisation();
  double dist = dm->normalisation();
  double ps = nm->produitScalaire(dm);
  delete(nm);
  delete(dm);
  if ( ps < 0.0 ) {
    e->r = 0.0;
    e->v = 0.0;
    e->b = 0.0; }
    else {
    e->r = intensite*c->r*ps*kd->r;
    e->v = intensite*c->v*ps*kd->v;
    e->b = intensite*c->b*ps*kd->b;
    if ( attenuation ) {
      e->r /= (dist*dist);
      e->v /= (dist*dist);
      e->b /= (dist*dist); } }
}

//////////////////////////////////////////////

int LumierePonctuelle::eclaire(Pos3D *p,Scene *scn,double dist) {
  Dir3D *dir = new Dir3D(p,this->p);
  dir->normalisation();
  RayonLumineux *rlo = new RayonLumineux(p,dir);
  int res = !scn->intersection(rlo,dist);
  delete(dir);
  delete(rlo);
  return(res);
}

/* Methode d'affichage texte                */
void LumierePonctuelle::print(void) {
  Lumiere::print();
  printf(" - ");
  p->print();
  printf(" - %d",attenuation);
}

double LumierePonctuelle::distance(Pos3D *p) {
  return(p->distance(this->p));
}

void LumierePonctuelle::transforme(TG3D *m) {
  p->transformation(m);
}

void LumierePonctuelle::modeliseOpenGL(int num) {
  float pos[4] = { p->x,p->y,p->z,1.0F };
  glLightfv(num,GL_POSITION,pos);
}

LumierePonctuelle *LumierePonctuelle::clone(void) {
  return(new LumierePonctuelle(this));
}
