/**
 * \file FileType.h
 * Classe de gestion des types de fichier de donnees.
 *
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 01/01/10
 */

#ifndef ____FILETYPE____
#define ____FILETYPE____

/**
 * \class FileType
 * Classe de gestion des types de fichier de donnees.
 *
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 01/01/10
 */

class FileType {

  public :
 
/**
 * \enum type_format_fichier
 * Valeurs possibles de definition de format de fichier 3D.
 * 
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 01/01/10
 */ 
    enum type_format_fichier {
      FORMAT_PERSONNEL, /**< Format proprietaire. */
      DXF,              /**< Format DXF (Autocad). */
      VRML,             /**< Format VRML. */
      X3D               /**< Format X3D. */
    };
 
/**
 * \enum type_format_fichier_donnees
 * Valeurs possibles de definition de format donnee numerique.
 * 
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 01/01/10
 */ 
    enum type_format_fichier_donnees {
      ENTIERS,          /**< Donnees sauvegardees en valeur entiere. */
      REELS             /**< Donnees sauvegardees en valeur reelle. */
    };
 
/**
 * \enum type_format_fichier_texte
 * Valeurs possibles de definition de format de fichier texte suivant le systeme d'exploitation.
 * 
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 01/01/10
 */ 
    enum type_format_fichier_texte {
      DOS,              /**< Fichier DOS. */
      UNIX,             /**< Fichier UNIX. */
      MAC               /**< Fichier Macintosh. */
    };

/**
 * Retour d'une chaine de caracteres utilisable pour realiser un retour chariot.
 *
 * @param systeme Le format de fichier texte de sauvegarde.
 * @author Nicolas Janey
 * @author nicolas.janey@univ-fcomte.fr
 * @version 1.0, 01/01/10
 */
    static char *retour_chariot(type_format_fichier_texte systeme) ;
};

#endif
